package org.wcs.mapper.intf;

import com.baomidou.mybatisplus.extension.service.IService;
import org.wcs.model.bo.tuple.Tuple2;
import org.wcs.model.dto.serve.stackerLocation.QueryStackerLocationReq;
import org.wcs.model.po.app.AppStackerLocation;

import java.util.List;

public interface AppStackerLocationService extends IService<AppStackerLocation> {

    Tuple2<Long, List<AppStackerLocation>> queryStackerLocation(QueryStackerLocationReq request, Integer pageSize, Integer pageIndex);
    List<AppStackerLocation> queryStackerLocation(QueryStackerLocationReq request);
    int insert(AppStackerLocation appStackerLocation); // 添加一个库位
    List<AppStackerLocation> queryStackerLocationById(String locationId); // 根据库位ID查询
    List<AppStackerLocation> queryStackerLocationAll(); // 查询所有
    boolean deleteWithMachineIdAndLaneIdNoCatchException(Integer machineId, Integer laneId, Integer rowId); // 根据堆垛机ID和巷道ID和排ID删除
    boolean insertNoCatchException(List<AppStackerLocation> appStackerLocations); // 批量添加
    boolean updateById(AppStackerLocation appStackerLocation); // 根据主键更新
    boolean updateStatusAndVehicleNoById(String locationId, Integer status, String vehicleNo); // 根据主键更新状态和载具
    List<AppStackerLocation> queryStackerLocationByStackerIdAndLaneId(Integer stackerId, Integer laneId);



}
